# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django import template
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from sahara_dashboard.api import sahara as saharaclient


class EditTagsAction(tables.LinkAction):
    name = "edit_tags"
    verbose_name = _("Edit Tags")
    url = "horizon:project:data_processing.clusters:edit_tags"
    classes = ("ajax-modal",)


def tags_to_string(image):
    template_name = 'image_registry/_list_tags.html'
    context = {"image": image}
    return template.loader.render_to_string(template_name, context)


class RegisterImage(tables.LinkAction):
    name = "register"
    verbose_name = _("Register Image")
    url = "horizon:project:data_processing.clusters:register"
    classes = ("ajax-modal",)
    icon = "plus"


class UnregisterImages(tables.DeleteAction):
    name = "unregister"

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Unregister Image",
            u"Unregister Images",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Unregistered Image",
            u"Unregistered Images",
            count
        )

    def delete(self, request, obj_id):
        saharaclient.image_unregister(request, obj_id)


class ImageRegistryTable(tables.DataTable):
    name = tables.Column("name",
                         verbose_name=_("Image"),
                         link=("horizon:project:"
                               "images:images:detail"))
    tags = tables.Column(tags_to_string,
                         verbose_name=_("Tags"))
    user = tables.Column("username", verbose_name=_("User"))

    class Meta(object):
        name = "image_registry"
        verbose_name = _("Image Registry")
        table_actions = (RegisterImage, UnregisterImages,)
        row_actions = (EditTagsAction, UnregisterImages,)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_dashboard.test.integration_tests.helpers import SaharaTestCase


PLUGIN_NAME = 'Fake Plugin'
PLUGIN_VERSION = '0.1'

IMAGE_TAGS = ('fake', PLUGIN_VERSION)


class TestCRUDBase(SaharaTestCase):

    def setUp(self):
        super(TestCRUDBase, self).setUp()
        self.flavor_name = self.gen_name('flavor')
        self.image_name = self.gen_name("image")
        self.master_name = self.gen_name("master")
        self.worker_name = self.gen_name("worker")
        self.template_name = self.gen_name("template")
        self.cluster_name = self.gen_name("cluster")
        self.ds_input_name = self.gen_name('input')
        self.ds_output_name = self.gen_name('output')
        self.job_binary_name = self.gen_name('function')
        self.jobtemplate_name = self.gen_name('test-job')

    def create_flavor(self):
        flavors_page = self.home_pg.go_to_system_flavorspage()

        flavors_page.create_flavor(
            name=self.flavor_name,
            vcpus=self.CONFIG.sahara.flavor_vcpus,
            ram=self.CONFIG.sahara.flavor_ram,
            root_disk=self.CONFIG.sahara.flavor_root_disk,
            ephemeral_disk=self.CONFIG.sahara.flavor_ephemeral_disk,
            swap_disk=self.CONFIG.sahara.flavor_swap_disk)
        self.assertTrue(flavors_page.is_flavor_present(self.flavor_name))

    def delete_flavor(self):
        flavors_page = self.home_pg.go_to_system_flavorspage()
        flavors_page.delete_flavor_by_row(self.flavor_name)
        self.assertFalse(flavors_page.is_flavor_present(self.flavor_name))

    def create_image(self):
        image_pg = self.home_pg.go_to_compute_imagespage()
        image_pg.create_image(
            self.image_name, image_file=self.CONFIG.sahara.fake_image_location)
        image_pg._wait_until(
            lambda x: image_pg.is_image_active(self.image_name),
            timeout=10 * 60)

    def delete_image(self):
        image_pg = self.home_pg.go_to_compute_imagespage()
        image_pg.delete_image(self.image_name)

    def register_image(self):
        image_reg_pg = (
            self.home_pg.go_to_dataprocessing_clusters_imageregistrypage())
        image_ssh_user = self.CONFIG.sahara.fake_image_ssh_user
        image_reg_pg.register_image(self.image_name, image_ssh_user,
                                    "Test description", tags=IMAGE_TAGS)
        image_reg_pg.wait_until_image_registered(self.image_name)

    def unregister_image(self):
        image_reg_pg = (
            self.home_pg.go_to_dataprocessing_clusters_imageregistrypage())
        image_reg_pg.unregister_image(self.image_name)

    def create_cluster(self):
        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )

        nodegrouptpls_pg.create(
            PLUGIN_NAME, PLUGIN_VERSION, nodegroup_name=self.worker_name,
            floating_ip_pool=self.CONFIG.sahara.ip_pool,
            flavor=self.flavor_name, processes=['tasktracker', 'datanode'])
        self.assertTrue(nodegrouptpls_pg.has_success_message())
        self.assertFalse(nodegrouptpls_pg.has_error_message())
        self.assertTrue(nodegrouptpls_pg.is_present(self.worker_name),
                        "Worker template was not created.")

        clustertpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_clustertemplatespage())

        clustertpls_pg.create(
            PLUGIN_NAME, PLUGIN_VERSION, name=self.template_name,
            node_group_templates=[self.master_name, self.worker_name])
        self.assertTrue(clustertpls_pg.has_success_message())
        self.assertFalse(clustertpls_pg.has_error_message())
        self.assertTrue(clustertpls_pg.is_present(self.template_name),
                        "Cluster template was not created.")

        cluster_pg = self.home_pg.go_to_dataprocessing_clusters_clusterspage()
        cluster_pg.create(PLUGIN_NAME, PLUGIN_VERSION, name=self.cluster_name,
                          image=self.image_name,
                          cluster_template=self.template_name)
        self.assertTrue(cluster_pg.has_success_message())
        self.assertFalse(cluster_pg.has_error_message())
        cluster_pg.refresh_page()
        self.assertTrue(cluster_pg.is_present(self.cluster_name),
                        "Cluster was not created.")

        cluster_pg.wait_until_cluster_active(
            self.cluster_name, timeout=self.CONFIG.sahara.launch_timeout)
        self.assertTrue(cluster_pg.is_cluster_active(self.cluster_name),
                        "Cluster is not active")

    def cluster_scale(self):
        cluster_pg = self.home_pg.go_to_dataprocessing_clusters_clusterspage()
        cluster_pg.scale(self.cluster_name, self.worker_name, 2)
        self.assertTrue(cluster_pg.has_success_message())
        self.assertFalse(cluster_pg.has_error_message())

        cluster_pg.wait_until_cluster_active(
            self.cluster_name, timeout=self.CONFIG.sahara.launch_timeout)
        self.assertTrue(cluster_pg.is_cluster_active(self.cluster_name),
                        "Cluster is not active")
        self.assertEqual(
            cluster_pg.get_cluster_instances_count(self.cluster_name), 2,
            "Cluster was not scaled")

    def create_datasources(self):
        datasource_pg = (
            self.home_pg.go_to_dataprocessing_jobs_datasourcespage())
        datasource_pg.create(name=self.ds_input_name, source_type="HDFS",
                             url="hdfs://user/input")
        datasource_pg.create(name=self.ds_output_name, source_type="Swift",
                             url="swift://container/output")

    def delete_datasources(self):
        datasource_pg = (
            self.home_pg.go_to_dataprocessing_jobs_datasourcespage())
        datasource_pg.delete(self.ds_input_name)
        datasource_pg.delete(self.ds_output_name)

    def create_job_binary(self):
        job_binary_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobbinariespage())
        job_binary_pg.create_job_binary_from_file(
            self.job_binary_name, __file__)

        self.assertTrue(
            job_binary_pg.is_job_binary_present(self.job_binary_name),
            "Job binary is not in the binaries job table after its creation.")

    def delete_job_binary(self):
        job_binary_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobbinariespage())
        job_binary_pg.delete_job_binary(self.job_binary_name)
        self.assertFalse(
            job_binary_pg.is_job_binary_present(self.job_binary_name),
            "Job binary was not removed from binaries job table.")

    def create_job_template(self):
        jobtemplates_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobtemplatespage())
        jobtemplates_pg.create(name=self.jobtemplate_name, job_type='Pig',
                               binary_name=self.job_binary_name)

    def delete_job_template(self):
        jobtemplates_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobtemplatespage())
        jobtemplates_pg.delete(self.jobtemplate_name)

    def run_edp_job(self):
        jobtemplates_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobtemplatespage())
        jobtemplates_pg.launch_on_exists(job_name=self.jobtemplate_name,
                                         input_name=self.ds_input_name,
                                         output_name=self.ds_output_name,
                                         cluster_name=self.cluster_name)
        jobs_pg = self.home_pg.go_to_dataprocessing_jobs_jobspage()
        jobs_pg.wait_until_job_succeeded(self.jobtemplate_name)

    def run_edp_job_with_parameters(self):
        jobtemplates_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobtemplatespage())
        jobtemplates_pg.launch_on_exists(job_name=self.jobtemplate_name,
                                         input_name=self.ds_input_name,
                                         output_name=self.ds_output_name,
                                         cluster_name=self.cluster_name,
                                         adapt_swift=False,
                                         datasource_substitution=False,
                                         configuration={'foo': 'bar'},
                                         parameters={'key': 'value'},
                                         arguments=('one', 'two'))
        jobs_pg = self.home_pg.go_to_dataprocessing_jobs_jobspage()
        details = jobs_pg.get_details(self.jobtemplate_name)
        expected = {
            'Job Configuration': {
                'configs': set([
                    'edp.substitute_data_source_for_name = False',
                    'foo = bar',
                    'edp.substitute_data_source_for_uuid = False'
                ]),
                'params': set(['key = value']),
                'args': set(['one', 'two']),
                'job_execution_info': set([]),
            },
            'Job Template': self.jobtemplate_name,
        }

        details = {k: v for k, v in details.items() if k in expected}
        self.assertEqual(expected, details)

    def delete_job(self):
        jobs_pg = self.home_pg.go_to_dataprocessing_jobs_jobspage()
        jobs_pg.delete(self.jobtemplate_name)

    def delete_cluster(self):
        cluster_pg = self.home_pg.go_to_dataprocessing_clusters_clusterspage()
        cluster_pg.delete(self.cluster_name)
        self.assertTrue(cluster_pg.has_success_message())
        self.assertFalse(cluster_pg.has_error_message())
        cluster_pg.wait_until_cluster_deleted(self.cluster_name)
        self.assertFalse(cluster_pg.is_present(self.cluster_name),
                         "Cluster was not deleted.")

    def tearDown(self):
        clustertpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_clustertemplatespage())
        if clustertpls_pg.is_present(self.template_name):
            clustertpls_pg.delete(self.template_name)

        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )
        nodegrouptpls_pg.delete_many((self.worker_name, self.master_name))
        super(TestCRUDBase, self).tearDown()


class TestCRUD(TestCRUDBase):

    def setUp(self):
        super(TestCRUD, self).setUp()
        self.create_flavor()
        self.create_image()
        self.register_image()

    def run_many_edp_jobs(self):
        jobtemplates_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobtemplatespage())
        job_names = {}
        for job_type in ("Hive", "Java", "MapReduce", "Streaming MapReduce",
                         "Pig", "Shell", "Spark", "Storm"):
            job_name = "{0}-{1}".format(self.jobtemplate_name, job_type)
            job_name = job_name.replace(' ', '-')
            job_names[job_type] = job_name
            binary_name = self.job_binary_name
            libs = []
            if job_type in ("Java", "MapReduce", "Streaming MapReduce"):
                binary_name = None
                libs.append(self.job_binary_name)
            jobtemplates_pg.create(name=job_name, job_type=job_type,
                                   binary_name=binary_name, libs=libs)

        for job_type, job_name in job_names.items():
            jobtemplates_pg = (
                self.home_pg.go_to_dataprocessing_jobs_jobtemplatespage())
            input_name = self.ds_input_name
            output_name = self.ds_output_name
            mapper = None
            reducer = None
            if job_type in ("Java", "Shell", "Storm", "Spark"):
                input_name = None
                output_name = None
            if job_type == "Streaming MapReduce":
                mapper = "mapper"
                reducer = "reducer"
            jobtemplates_pg.launch_on_exists(job_name=job_name,
                                             input_name=input_name,
                                             output_name=output_name,
                                             cluster_name=self.cluster_name,
                                             mapper=mapper,
                                             reducer=reducer)
            jobs_pg = self.home_pg.go_to_dataprocessing_jobs_jobspage()
            jobs_pg.wait_until_job_succeeded(job_name)

        jobs_pg.delete_many(job_names.values())

        jobtemplates_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobtemplatespage())
        jobtemplates_pg.delete_many(job_names.values())

    def test_cluster_operate(self):
        self.create_cluster()
        self.create_datasources()
        self.create_job_binary()
        self.create_job_template()
        self.run_edp_job()
        self.delete_job()
        self.run_edp_job_with_parameters()
        self.delete_job()
        self.run_many_edp_jobs()
        self.delete_job_template()
        self.delete_job_binary()
        self.delete_datasources()
        self.cluster_scale()
        self.delete_cluster()

    def tearDown(self):
        self.unregister_image()
        self.delete_image()
        self.delete_flavor()
        super(TestCRUD, self).tearDown()


class TestUpdateNodeGroupTemplate(SaharaTestCase):
    def setUp(self):
        super(TestUpdateNodeGroupTemplate, self).setUp()
        self.old_name = self.gen_name("old-name")
        self.new_name = self.gen_name("new-name")
        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )

        nodegrouptpls_pg.create(
            PLUGIN_NAME, PLUGIN_VERSION, nodegroup_name=self.old_name,
            flavor='m1.tiny', processes=['tasktracker', 'datanode'])
        self.assertTrue(nodegrouptpls_pg.has_success_message())
        self.assertFalse(nodegrouptpls_pg.has_error_message())
        self.assertTrue(nodegrouptpls_pg.is_present(self.old_name),
                        "Worker template was not created.")

    def test_update(self):
        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )
        kwargs = {
            'nodegroup_name': self.new_name,
            'description': '{} description'.format(self.new_name),
            'flavor': 'm1.small',
            'availability_zone': 'No availability zone specified',
            'storage': 'Cinder Volume',
            'volumes_per_node': 2,
            'volumes_size': 3,
            'volume_type': 'lvmdriver-1',
            'volume_local_to_instance': True,
            'volumes_availability_zone': 'nova',
            'floating_ip_pool': self.CONFIG.sahara.ip_pool,
            'use_autoconfig': False,
            'proxygateway': True,
            'processes': ['jobtracker', 'namenode'],
            'security_autogroup': False,
            'security_groups': ['default']
        }
        nodegrouptpls_pg.update(group_name=self.old_name, **kwargs)
        details = nodegrouptpls_pg.get_details(self.new_name)
        expected = {
            'Auto Security Group': 'False',
            'Description': kwargs['description'],
            'Flavor': 'm1.small',
            'Floating IP Pool': self.CONFIG.sahara.ip_pool,
            'Name': self.new_name,
            'Node Processes': 'namenode\njobtracker',
            'Plugin': 'fake',
            'Proxy Gateway': 'True',
            'Security Groups': 'default',
            'Use auto-configuration': 'False',
            'Version': '0.1',
            'Volumes Availability Zone': 'nova',
            'Volumes local to instance': 'True',
            'Volumes per node': '2',
            'Volumes size': '3',
            'Volumes type': 'lvmdriver-1'
        }
        details = {k: v for k, v in details.items() if k in expected}
        self.assertEqual(expected, details)

    def tearDown(self):
        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )
        nodegrouptpls_pg.delete_many((self.old_name, self.new_name))
        super(TestUpdateNodeGroupTemplate, self).tearDown()


class TestUpdateClusterTemplate(SaharaTestCase):
    def setUp(self):
        super(TestUpdateClusterTemplate, self).setUp()
        self.flavor_name = self.gen_name('flavor')
        self.image_name = self.gen_name("image")

        self.worker_name = self.gen_name("worker")
        self.new_worker_name = self.gen_name("new-worker")
        self.master_name = self.gen_name("master")
        self.cluster_template_name = self.gen_name("old-name")
        self.new_cluster_template_name = self.gen_name("new-name")

        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )

        nodegrouptpls_pg.create(
            PLUGIN_NAME, PLUGIN_VERSION, nodegroup_name=self.worker_name,
            flavor='m1.tiny', processes=['tasktracker', 'datanode'])
        self.assertTrue(nodegrouptpls_pg.has_success_message())
        self.assertFalse(nodegrouptpls_pg.has_error_message())
        self.assertTrue(nodegrouptpls_pg.is_present(self.worker_name),
                        "Worker template was not created.")
        nodegrouptpls_pg.create(
            PLUGIN_NAME, PLUGIN_VERSION, nodegroup_name=self.master_name,
            flavor='m1.tiny', proxygateway=True,
            floating_ip_pool=self.CONFIG.sahara.ip_pool,
            processes=['jobtracker', 'namenode'])
        self.assertTrue(nodegrouptpls_pg.has_success_message())
        self.assertFalse(nodegrouptpls_pg.has_error_message())
        self.assertTrue(nodegrouptpls_pg.is_present(self.master_name),
                        "Worker template was not created.")
        clustertpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_clustertemplatespage())

        clustertpls_pg.create(
            PLUGIN_NAME, PLUGIN_VERSION, name=self.cluster_template_name,
            node_group_templates=[self.worker_name])
        self.assertTrue(clustertpls_pg.has_success_message())
        self.assertFalse(clustertpls_pg.has_error_message())
        self.assertTrue(clustertpls_pg.is_present(self.cluster_template_name),
                        "Cluster template was not created.")

    def test_update(self):
        clustertpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_clustertemplatespage())
        kwargs = {
            'cluster_template_name': self.new_cluster_template_name,
            'description': '{} description'.format(
                self.new_cluster_template_name),
            'use_autoconfig': False,
            'anti_affinity': ['namenode', 'datanode', 'tasktracker',
                              'jobtracker'],
            'node_group_templates': [self.master_name],
            'CONF:general:Timeout for disk preparing': 152,
            'CONF:general:Enable NTP service': False,
            'CONF:general:URL of NTP server': 'http://ntp.org/',
            'CONF:general:Heat Wait Condition timeout': 123,
            'CONF:general:Enable XFS': False,
        }
        clustertpls_pg.update(name=self.cluster_template_name, **kwargs)
        self.assertTrue(clustertpls_pg.has_success_message())
        self.assertFalse(clustertpls_pg.has_error_message())
        self.assertTrue(
            clustertpls_pg.is_present(self.new_cluster_template_name),
            "Cluster template was not updated.")
        details = clustertpls_pg.get_details(self.new_cluster_template_name)

        expected = {
            'Name': self.new_cluster_template_name,
            'Description': kwargs['description'],
            'Plugin': 'fake',
            'Version': '0.1',
            'Use auto-configuration': 'False',
            'Anti-affinity enabled for': set(['namenode', 'datanode',
                                              'tasktracker', 'jobtracker']),
            'Enable NTP service': 'False',
            'Enable XFS': 'False',
            'Heat Wait Condition timeout': '123',
            'Timeout for disk preparing': '152',
            'URL of NTP server': 'http://ntp.org/',
            'node_groups': {
                self.master_name: '1'
            }
        }
        details = {k: v for k, v in details.items() if k in expected}
        self.assertEqual(expected, details)

    def test_update_nodegroup(self):
        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )
        kwargs = {
            'nodegroup_name': self.new_worker_name,
            'description': '{} description'.format(self.new_worker_name),
            'flavor': 'm1.small',
            'availability_zone': 'No availability zone specified',
            'floating_ip_pool': 'public',
            'use_autoconfig': False,
            'proxygateway': True,
            'processes': ['jobtracker', 'namenode'],
            'security_autogroup': False,
            'security_groups': ['default']
        }
        nodegrouptpls_pg.update(group_name=self.worker_name, **kwargs)
        self.assertTrue(nodegrouptpls_pg.has_success_message())
        self.assertFalse(nodegrouptpls_pg.has_error_message())
        self.assertTrue(nodegrouptpls_pg.is_present(self.new_worker_name),
                        "Node group template was not updated.")
        clustertpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_clustertemplatespage())
        details = clustertpls_pg.get_nodegroup_details(
            self.cluster_template_name, self.new_worker_name)
        expected = {
            'Auto Security Group': 'no',
            'Flavor': '2',
            'Node Processes': {'namenode', 'jobtracker'},
            'Nodes Count': '1',
            'Proxy Gateway': 'yes',
            'Security Groups': 'default',
            'Template': self.new_worker_name,
            'Use auto-configuration': 'False'
        }
        details = {k: v for k, v in details.items() if k in expected}
        self.assertEqual(expected, details)

    def tearDown(self):
        clustertpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_clustertemplatespage())
        clustertpls_pg.delete_many((self.cluster_template_name,
                                    self.new_cluster_template_name))

        nodegrouptpls_pg = (
            self.home_pg.go_to_dataprocessing_clusters_nodegrouptemplatespage()
        )
        nodegrouptpls_pg.delete_many((self.worker_name, self.master_name,
                                      self.new_worker_name))
        super(TestUpdateClusterTemplate, self).tearDown()


class TestUpdateDataSource(SaharaTestCase):

    def setUp(self):
        super(TestUpdateDataSource, self).setUp()
        datasource_pg = (
            self.home_pg.go_to_dataprocessing_jobs_datasourcespage())
        self.name = self.gen_name('input')
        self.new_name = '{}-new'.format(self.name)
        datasource_pg.create(name=self.name, source_type="HDFS",
                             url="hdfs://user/input")

    def test_update(self):
        datasource_pg = (
            self.home_pg.go_to_dataprocessing_jobs_datasourcespage())
        kwargs = {
            'data_source_name': self.new_name,
            'data_source_type': 'Swift',
            'data_source_url': 'swift://container.sahara/object',
            'data_source_credential_user': 'test-user',
            'data_source_credential_pass': 'test-password',
            'data_source_description': '{} description'.format(self.new_name),
        }
        datasource_pg.update(self.name, **kwargs)
        self.assertTrue(datasource_pg.has_success_message())
        self.assertFalse(datasource_pg.has_error_message())
        self.assertTrue(datasource_pg.is_present(self.new_name),
                        "Node group template was not updated.")
        details = datasource_pg.get_details(self.new_name)
        expected = {
            'Description': kwargs['data_source_description'],
            'Name': self.new_name,
            'Type': 'swift',
            'URL': 'swift://container.sahara/object'
        }
        details = {k: v for k, v in details.items() if k in expected}
        self.assertEqual(expected, details)

    def tearDown(self):
        datasource_pg = (
            self.home_pg.go_to_dataprocessing_jobs_datasourcespage())
        datasource_pg.delete_many([self.name, self.new_name])
        super(TestUpdateDataSource, self).tearDown()


class TestUpdateJobBinaries(SaharaTestCase):

    def setUp(self):
        super(TestUpdateJobBinaries, self).setUp()
        # create job binary
        job_binary_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobbinariespage())
        self.name = self.gen_name('function')
        self.new_name = '{}-new'.format(self.name)
        job_binary_pg.create_job_binary_from_file(self.name, __file__)

    def test_update(self):
        job_binary_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobbinariespage())
        kwargs = {
            "job_binary_name": self.new_name,
            "job_binary_type": "Internal database",
            "job_binary_internal": "*Create a script",
            "job_binary_script_name": "script-name",
            "job_binary_script": "script-text",
            "job_binary_description": "{} description".format(self.new_name),
        }
        job_binary_pg.update_job_binary(self.name, **kwargs)
        self.assertTrue(job_binary_pg.has_success_message())
        self.assertFalse(job_binary_pg.has_error_message())
        self.assertTrue(job_binary_pg.is_job_binary_present(self.new_name),
                        "Job binary was not updated.")
        details = job_binary_pg.get_details(self.new_name)
        expected = {
            'Description': kwargs['job_binary_description'],
            'Name': self.new_name,
        }

        details = {k: v for k, v in details.items() if k in expected}
        self.assertEqual(expected, details)

    def tearDown(self):
        job_binary_pg = (
            self.home_pg.go_to_dataprocessing_jobs_jobbinariespage())
        for name in (self.name, self.new_name):
            try:
                job_binary_pg.delete_job_binary(name)
            except Exception:
                pass
        super(TestUpdateJobBinaries, self).tearDown()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.pages import basepage
from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables


class ImageRegistryTable(tables.TableRegion):
    name = "image_registry"
    REGISTER_FORM_FIELDS = ('image_id', 'user_name', 'description')

    @tables.bind_table_action('register')
    def register_image(self, register_button):
        register_button.click()
        return forms.FormRegion(self.driver, self.conf,
                                field_mappings=self.REGISTER_FORM_FIELDS)

    @tables.bind_table_action('unregister')
    def unregister_image(self, unregister_button):
        unregister_button.click()
        return forms.BaseFormRegion(self.driver, self.conf)


class ImageregistryPage(basepage.BaseNavigationPage):

    TABLE_IMAGE_COLUMN = 'name'

    def __init__(self, driver, conf):
        super(ImageregistryPage, self).__init__(driver, conf)
        self._page_title = "Clusters"

    def _get_row_with_image_name(self, name):
        return self.image_table.get_row(self.TABLE_IMAGE_COLUMN, name)

    @property
    def image_table(self):
        return ImageRegistryTable(self.driver, self.conf)

    def is_image_registered(self, name):
        return bool(self._get_row_with_image_name(name))

    def unregister_image(self, name):
        self._get_row_with_image_name(name).mark()
        unregister_form = self.image_table.unregister_image()
        unregister_form.submit()

    def register_image(self, image, user_name, description, tags=()):
        register_form = self.image_table.register_image()
        register_form.image_id.text = image
        register_form.user_name.text = user_name
        register_form.description.text = description
        tag_input = register_form.src_elem.find_element_by_id(
            "_sahara_image_tag")
        tag_add_btn = register_form.src_elem.find_element_by_id("add_tag_btn")
        for tag in tags:
            tag_input.send_keys(tag)
            tag_add_btn.click()
        register_form.submit()

    def wait_until_image_registered(self, name):
        self._wait_until(lambda x: self.is_image_registered(name))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables

from sahara_dashboard.test.integration_tests.pages import basepage
from sahara_dashboard.test.integration_tests.pages import mixins


CREATE_FIELD_MAPPING = (
    ('nodegroup_name', 'description', 'flavor', 'availability_zone', 'storage',
        'volumes_per_node', 'volumes_size', 'volume_type',
        'volume_local_to_instance', 'volumes_availability_zone',
        'floating_ip_pool', 'use_autoconfig', 'proxygateway'),
    ('processes',),
    ('security_autogroup', 'security_groups'),
)


class UpdateMixin(object):
    @tables.bind_row_action('edit')
    def get_edit_form(self, button, row):
        button.click()
        return forms.TabbedFormRegion(
            self.driver, self.conf,
            field_mappings=CREATE_FIELD_MAPPING)


class NodegrouptemplatesPage(mixins.PluginSelectMixin, mixins.DeleteMixin,
                             basepage.BaseDataProcessingPage):

    TABLE_NAME = "nodegroup_templates"

    @classmethod
    def get_table_mixins(cls):
        return (super(NodegrouptemplatesPage, cls).get_table_mixins() +
                (UpdateMixin,))

    @property
    def create_form(self):
        return forms.TabbedFormRegion(
            self.driver, self.conf, field_mappings=CREATE_FIELD_MAPPING)

    def _set_checkbox_group(self, form, group_name, values=()):
        for el in form.src_elem.find_elements_by_xpath(
            './/input[@name="{}"]'.format(group_name)
        ):
            elem_id = el.get_attribute('id')
            label_text = form.src_elem.find_element_by_css_selector(
                'label[for={}]'.format(elem_id)).text
            if (label_text in values) != el.is_selected():
                el.click()

    def _fill_form(self, form, **kwargs):
        for tab_num, fields in enumerate(CREATE_FIELD_MAPPING):
            form.switch_to(tab_num)
            for key in fields:
                value = kwargs.get(key)
                if value is None:
                    continue
                if isinstance(value, (list, tuple)):
                    self._set_checkbox_group(form, key, value)
                    continue
                field = getattr(form, key, None)
                if (field.src_elem.get_attribute('type') in
                        ('checkbox', 'radio')):
                    if value:
                        field.mark()
                    else:
                        field.unmark()
                else:
                    field.text = value

    def create(self, plugin_name, plugin_version, nodegroup_name, flavor,
               floating_ip_pool=None, availability_zone='nova',
               proxygateway=False, processes=(), **kwargs):
        self.choose_plugin(plugin_name, plugin_version)
        form = self.create_form
        kwargs.update({
            'nodegroup_name': nodegroup_name,
            'flavor': flavor,
            'floating_ip_pool': floating_ip_pool,
            'availability_zone': availability_zone,
            'proxygateway': proxygateway,
            'processes': processes,
        })
        self._fill_form(form, **kwargs)
        form.submit()

    def update(self, group_name, **kwargs):
        row = self._get_row_with_name(group_name)
        form = self.table.get_edit_form(row)
        self._fill_form(form, **kwargs)
        form.submit()

    def get_details(self, name):
        self.table.src_elem.find_element_by_link_text(name).click()
        items = self.driver.find_elements_by_css_selector('div.detail > dl')
        self._turn_off_implicit_wait()
        details = {}
        for item in items:
            names = [x.text for x in item.find_elements_by_tag_name('dt')]
            values = [x.text for x in item.find_elements_by_tag_name('dd')]
            details.update(zip(names, values))
        self._turn_on_implicit_wait()
        return details



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.pages import basepage
from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import messages
from openstack_dashboard.test.integration_tests.regions import tables


class JobBinariesTable(tables.TableRegion):
    name = 'job_binaries'
    CREATE_BINARY_FORM_FIELDS = (
        "job_binary_name",
        "job_binary_type",
        "job_binary_url",
        "job_binary_username",
        "job_binary_password",
        "job_binary_internal",
        "job_binary_file",
        "job_binary_script_name",
        "job_binary_script",
        "job_binary_description"
    )

    @tables.bind_table_action('create')
    def get_create_form(self, button):
        button.click()
        return forms.FormRegion(
            self.driver, self.conf,
            field_mappings=self.CREATE_BINARY_FORM_FIELDS)

    @tables.bind_table_action('delete')
    def get_delete_form(self, button):
        button.click()
        return forms.BaseFormRegion(self.driver, self.conf)

    @tables.bind_row_action('edit_job_binary')
    def get_update_form(self, button, row):
        button.click()
        return forms.FormRegion(
            self.driver, self.conf,
            field_mappings=self.CREATE_BINARY_FORM_FIELDS)


class JobbinariesPage(basepage.BaseNavigationPage):

    JOB_BINARIES_TABLE_NAME_COLUMN = 'name'

    def __init__(self, driver, conf):
        super(JobbinariesPage, self).__init__(driver, conf)
        self._page_title = "Data Processing"

    def _get_row_with_name(self, name):
        return self.table.get_row(
            self.JOB_BINARIES_TABLE_NAME_COLUMN, name)

    @property
    def table(self):
        return JobBinariesTable(self.driver, self.conf)

    def delete_job_binary(self, name):
        row = self._get_row_with_name(name)
        row.mark()
        confirm_delete_form = self.table.get_delete_form()
        confirm_delete_form.submit()

    def create_job_binary(self, binary_name, script_name):
        form = self.table.get_create_form()

        form.job_binary_name.text = binary_name
        form.job_binary_type.text = "Internal database"
        form.job_binary_internal.text = "*Create a script"
        form.job_binary_script_name.text = script_name
        form.job_binary_script.text = "test_script_text"
        form.job_binary_description.text = "test description"
        form.submit()

    def create_job_binary_from_file(self, binary_name, path):
        form = self.table.get_create_form()

        form.job_binary_name.text = binary_name
        form.job_binary_type.text = "Internal database"
        form.job_binary_internal.text = "*Upload a new file"
        form.job_binary_file.src_elem.send_keys(path)
        form.job_binary_description.text = "test description"
        form.submit()

    def update_job_binary(self, name, **kwargs):
        row = self._get_row_with_name(name)
        form = self.table.get_update_form(row)
        for key in JobBinariesTable.CREATE_BINARY_FORM_FIELDS:
            if key in kwargs:
                getattr(form, key).text = kwargs[key]
        form.submit()

    def get_details(self, name):
        details = {}
        self.table.src_elem.find_element_by_link_text(name).click()
        items = self.driver.find_elements_by_css_selector('div.detail dt')
        for item in items:
            key = item.text
            value_elem = item.find_element_by_xpath('./following-sibling::*')
            if value_elem.tag_name != "dd":
                continue
            value = value_elem.text
            details[key] = value
        return details

    def is_job_binary_present(self, name):
        return bool(self._get_row_with_name(name))

    def has_success_message(self):
        return self.find_message_and_dismiss(messages.SUCCESS)

    def has_error_message(self):
        return self.find_message_and_dismiss(messages.ERROR)
